'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Orchestra = undefined;

var _extends3 = require('babel-runtime/helpers/extends');

var _extends4 = _interopRequireDefault(_extends3);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _symbol = require('babel-runtime/core-js/symbol');

var _symbol2 = _interopRequireDefault(_symbol);

exports.default = createOrchestra;

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _rxjs = require('@reactivex/rxjs');

var _Collection = require('./Collection');

var _Store = require('./Store');

var _combineStores = require('./util/combineStores');

var _combineStores2 = _interopRequireDefault(_combineStores);

var _createGetter = require('./util/createGetter');

var _createGetter2 = _interopRequireDefault(_createGetter);

var _capitalizeString = require('./util/capitalizeString');

var _capitalizeString2 = _interopRequireDefault(_capitalizeString);

var _isDispatcher = require('fluorine-lib/lib/util/isDispatcher');

var _isDispatcher2 = _interopRequireDefault(_isDispatcher);

var _immutable = require('immutable');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var resultCache = (0, _symbol2.default)('resultCache');

var Orchestra = exports.Orchestra = function () {
  Orchestra.isOrchestra = function isOrchestra(obj) {
    return (typeof obj === 'undefined' ? 'undefined' : (0, _typeof3.default)(obj)) === 'object' && obj instanceof Orchestra;
  };

  function Orchestra() {
    (0, _classCallCheck3.default)(this, Orchestra);

    for (var _len = arguments.length, stores = Array(_len), _key = 0; _key < _len; _key++) {
      stores[_key] = arguments[_key];
    }

    (0, _invariant2.default)(stores.length > 0 && stores.every(_Store.Store.isStore), 'Orchestra: constructor expects to receive Stores.');

    var _stores = stores.reduce(function (acc, store) {
      var identifier = store.identifier;


      if (acc[identifier]) {
        throw new Error('Orchestra: The identifier `' + identifier + '` is not unique.');
      }

      acc[identifier] = store;
      return acc;
    }, {});

    this[resultCache] = {};
    this.stores = _stores;
    this.externals = {};
  }

  Orchestra.prototype.addReducer = function addReducer(identifier, reducer) {
    (0, _invariant2.default)(typeof identifier === 'string', 'Orchestra: `identifier` is expected to be a string.');
    (0, _invariant2.default)(typeof reducer === 'function', 'Orchestra: `reducer` is expected to be a reducer function.');

    var externals = this.externals;
    var stores = this.stores;


    (0, _invariant2.default)(!stores.hasOwnProperty(identifier), 'Orchesta: The identifier `' + identifier + '` is already taken by a Store.');
    (0, _invariant2.default)(!externals.hasOwnProperty(identifier), 'Orchesta: The identifier `' + identifier + '` is not unique.');

    externals[identifier] = reducer;
    return this;
  };

  Orchestra.prototype.reduce = function reduce(dispatcher) {
    (0, _invariant2.default)((0, _isDispatcher2.default)(dispatcher), 'Orchestra: `dispatcher` is expected to be a Fluorine dispatcher.');
    if (this[resultCache][dispatcher.identifier]) {
      return this[resultCache][dispatcher.identifier];
    }

    var stores = this.stores;
    var externals = this.externals;

    // Assemble reduced externals

    var _externals = (0, _keys2.default)(externals).reduce(function (acc, key) {
      var external = externals[key];

      acc[key] = dispatcher.reduce(external);

      return acc;
    }, {});

    var _stores = (0, _keys2.default)(stores).reduce(function (acc, key) {
      var store = stores[key];
      var reducer = store.getReducer();

      acc[key] = dispatcher.reduce(reducer);

      return acc;
    }, {});

    // Empty collections for convenience
    var _emptyCollections = (0, _keys2.default)(stores).reduce(function (acc, key) {
      var store = stores[key];
      var collection = store.createCollection();

      acc[key] = collection;
      return acc;
    }, {});

    var _loop = function _loop(identifier) {
      if (stores.hasOwnProperty(identifier)) {
        var _ret2 = function () {
          var _extends2;

          // Get the unresolved fluorine store
          var fluorineStore = _stores[identifier];

          // Get the post hook and all dependencies for this store
          var orchestraStore = stores[identifier];
          var post = orchestraStore.getPost();
          var dependencies = orchestraStore.getDependencies();
          var reducer = orchestraStore.getReducer();

          // If the store doesn't have any dependencies we only apply the post hook
          if ((0, _keys2.default)(dependencies).length === 0) {
            // Override the old fluorine store
            _stores[identifier] = fluorineStore.map(function (state) {
              return state.map(post);
            }).publishReplay(1).refCount();

            return {
              v: 'continue'
            }; // Honey, I'm home (early)
          }

          // Build object containing fluorine stores for all dependencies
          var _dependencyStores = (0, _keys2.default)(dependencies).reduce(function (acc, dependency) {
            if (_externals[dependency]) {
              acc[dependency] = _externals[dependency];
            } else if (_stores[dependency]) {
              acc[dependency] = _stores[dependency];
            } else {
              throw new Error('Orchestra: Failed to resolve dependency for identifier `' + dependency + '`.');
            }

            return acc;
          }, {});

          // Override the old fluorine store
          _stores[identifier] = (0, _combineStores2.default)((0, _extends4.default)({}, _dependencyStores, (_extends2 = {}, _extends2[identifier] = fluorineStore, _extends2))).map(function (deps) {
            return (0, _keys2.default)(dependencies).reduce(function (state, dependencyIdentifier) {
              // Retrieve the dependency getter, setter and its state
              var dependencyState = deps[dependencyIdentifier];
              var _dependencies$depende = dependencies[dependencyIdentifier];
              var getter = _dependencies$depende.getter;
              var setter = _dependencies$depende.setter;

              var emptyCollection = _emptyCollections[dependencyIdentifier];
              var dependencyName = (0, _capitalizeString2.default)(dependencyIdentifier);

              var missingIds = [];

              // Modify every item
              var _state = state.map(function (x) {
                // Get dependency ids
                var ids = getter(x, dependencyState);

                (0, _invariant2.default)(typeof ids === 'string' || _immutable.Iterable.isIterable(ids) || Array.isArray(ids), 'Orchestra: `ids` is expected to be a string, an iterable, or an Array.');

                // Collect missing ids from dependency item
                if (typeof ids === 'string') {
                  missingIds.push(ids);
                } else {
                  ids.forEach(function (id) {
                    if (!dependencyState.get(id)) {
                      missingIds.push(id);
                    }
                  });
                }

                var _get = (0, _createGetter2.default)(dependencyState, emptyCollection, ids);
                if (setter) {
                  return setter(x, _get);
                }

                x['get' + dependencyName] = _get;
                return x;
              });

              // Report missing ids
              var dependencyStore = stores[dependencyIdentifier];
              if (dependencyStore) {
                dependencyStore._missing(new _immutable.Set(missingIds), identifier);
              }

              return _state;
            }, deps[identifier]).map(post);
          }).publishReplay(1).refCount();
        }();

        if ((typeof _ret2 === 'undefined' ? 'undefined' : (0, _typeof3.default)(_ret2)) === "object") return _ret2.v;
      }
    };

    for (var identifier in stores) {
      var _ret = _loop(identifier);

      if (_ret === 'continue') continue;
    }

    this[resultCache][dispatcher.identifier] = _stores;
    return _stores;
  };

  return Orchestra;
}();

function createOrchestra() {
  for (var _len2 = arguments.length, stores = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
    stores[_key2] = arguments[_key2];
  }

  return new (Function.prototype.bind.apply(Orchestra, [null].concat(stores)))();
}