'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

exports.default = createReducerForStore;

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _Store = require('../Store');

var _toMap = require('./toMap');

var _toMap2 = _interopRequireDefault(_toMap);

var _immutable = require('immutable');

var _StoreConstants = require('../constants/StoreConstants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createReducerForStore(store) {
  (0, _invariant2.default)(store && _Store.Store.isStore(store), 'Reducer: `store` is expected to be an Orchestra.Store.');

  var identifier = store.getIdentifier();
  var pre = store.getPre();
  var initial = store.createCollection();

  return function storeReducer() {
    var state = arguments.length <= 0 || arguments[0] === undefined ? initial : arguments[0];
    var action = arguments[1];

    if (identifier !== action.identifier) {
      return state;
    }

    var type = action.type;
    var payload = action.payload;
    var selector = action.selector;
    var groupId = action.groupId;

    switch (type) {
      case _StoreConstants.STORE_INSERT:
        {
          if (_immutable.Map.isMap(payload)) {
            var item = pre(payload);
            if (!item) {
              return state;
            }

            var id = item.get('id');

            var res = state.set(id, item);
            if (groupId) {
              return res.addIdToGroup(groupId, id);
            }

            return res;
          }

          return state.withMutations(function (map) {
            // Deduping the incoming data by ids and tracking the ids
            var track = {};

            payload.forEach(function (value) {
              var item = pre(value);
              if (!item) {
                return state;
              }

              var id = item.get('id');
              if (!track[id]) {
                track[id] = true;
                map.set(id, item);
              }
            });

            if (groupId) {
              var ids = (0, _keys2.default)(track);
              map.addIdsToGroup(groupId, new _immutable.OrderedSet(ids));
            }
          });
        }

      case _StoreConstants.STORE_REMOVE:
        {
          if (!payload) {
            return state;
          } else if (_immutable.Map.isMap(payload)) {
            var _id = payload.get('id');
            return state.delete(_id);
          }

          return state.delete(payload);
        }

      case _StoreConstants.STORE_FILTER:
        {
          if (typeof selector !== 'function') {
            return state;
          }

          return state.filter(selector);
        }

      case _StoreConstants.STORE_UPDATE:
        {
          if (typeof selector !== 'function') {
            return state;
          }

          return state.map(function (x) {
            var item = pre(selector(x));
            return item ? item : x;
          });
        }

      default:
        return state;
    }
  };
}